import os.path

import pytest

from nbprint import Configuration
from nbprint.cli import run_hydra


def _example_folder_does_not_exist():
    return not os.path.exists("examples")


@pytest.mark.skipif(_example_folder_does_not_exist(), reason="Examples not present - skipping examples tests")
@pytest.mark.parametrize("template", ("basic", "inline", "landscape", "finance", "research", "plotly"))
def test_e2e(template):
    config = Configuration.load(f"examples/{template}.yaml", template)
    config.run()


@pytest.mark.skipif(_example_folder_does_not_exist(), reason="Examples not present - skipping examples tests")
@pytest.mark.parametrize("parameters", ("string1", "string2"))
def test_hydra_e2e(parameters):
    run_hydra("examples/hydra.yaml", ["config=inline", "page=report", f"parameters={parameters}"])



from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin


class NBPrintSearchPathPlugin(SearchPathPlugin):
    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        search_path.append(
            provider="nbprint-plugin", path="pkg://nbprint/config/hydra"
        )



from pathlib import Path

import pytest
from nbformat import reads

from nbprint.cli import run_hydra


@pytest.mark.parametrize("file", list((Path(__file__).parent / "files").glob("*")))
def test_integrations(file):
    res = run_hydra(str(file.resolve()))

    # file specific tests
    if file.name == "capture.yaml":
        output = res.outputs.resolve_output(res).read_text()
        nb = reads(output, as_version=4)
        assert nb.cells[-1].source == '%%capture\nprint("test1")\nimport logging\nlogging.critical("test!")\n'
        assert nb.cells[-1].outputs == []

    if file.name == "plugins.yaml":
        output = res.outputs.resolve_output(res).read_text()
        assert "Example Page Content from Plugin" in output



from pathlib import Path
from typing import Optional

from hydra import compose, initialize_config_dir
from hydra.utils import instantiate
from omegaconf import OmegaConf
from typer import Argument, Typer

from .config import Configuration

__all__ = ("cfg", "main", "run", "run_hydra")


def run(path: Path, name: str = "", dry_run: bool = False) -> Configuration:
    return run_hydra(path=path, overrides=name.split(" "), dry_run=dry_run)


def cfg(path: Path, name: str = "", dry_run: bool = False) -> Configuration:
    return run_hydra(path=path, overrides=name.split(" "), cfg=True, dry_run=dry_run)


def run_hydra(
    path: str,
    overrides: Optional[list[str]] = Argument(None),
    cfg: bool = False,
    dry_run: bool = False,
) -> Configuration:
    # convert to Path
    path = Path(path)

    if not isinstance(overrides, list):
        # maybe running via python, reset
        overrides = []

    # prune any empty strings
    overrides = [o for o in overrides if o]

    with initialize_config_dir(config_dir=str(path.parent.absolute()), version_base=None):
        hydra_config = compose(config_name=str(path.name), overrides=overrides)

        # bridge hydra and non-hydra
        extras = {"name": path.name.replace(".yaml", "").replace(".yml", "")} if "name" not in hydra_config else {}
        config = instantiate(hydra_config, **extras)
        if not isinstance(config, Configuration):
            config = Configuration(**config)

    # mimic hydra cfg
    if cfg:
        print(OmegaConf.to_yaml(config.model_dump(mode="json")))  # noqa: T201
    else:
        # TODO: return file?
        config.run(dry_run=dry_run)
    return config


def main() -> None:
    app = Typer()
    app.command("run")(run)
    app.command("cfg")(cfg)
    app.command("hydra")(run_hydra)
    app()
